//
// Charmed Labs
// www.charmedlabs.com
//
// This software is provided free of charge by Charmed Labs (CL) on an "as is"
// basis with no warranty expressed or implied.  CL grants permission to use
// this software but offers no guarantees regarding its usefulness toward any
// application.
// 
// redgreenmap.v 
// 
// Log:
// Revision 1.0: 7/16/02 by Rich LeGrand (rich@charmedlabs.com)
// Initial release
// 

module LogicMap(Addr, Data, En, Rd, Wr, Clk, Ireq, P);
     
   input  [23:0] Addr;
   inout  [15:0] Data; 
   input  En;
   input  Rd;
   input  Wr;
   input  Clk;
   output Ireq;
   inout  [54:0] P;  
     
   reg     [15:0] Data;
   reg 	   [54 :0] P;
     
   // The following two procedural blocks map a 16-bit I/O location at GBA
   // address 0x9ffe000
   
   // Handle write operation
   always @(negedge Wr)
      begin
      if (En)
         // transfer data bus bits to LED control signals (P[18] and P[19])     
         {P[19], P[18]} = {~Data[1], ~Data[0]}; 
      end
      
   // Handle read operation
   always @(En or Rd or P)
      begin
      if (Rd)
         begin
         if (En) 
            // transfer state of LED control signals to data bus
            Data = {14'b0, ~P[19], ~P[18]};
         end
      else
         // let flash use data bus
         Data = 16'bZ; 
      end
      
   // force interrupt request signal low
   assign Ireq = 1'b0;
      
endmodule
 
