//
// Charmed Labs
// www.charmedlabs.com
//
// This software is provided free of charge by Charmed Labs (CL) on an "as is"
// basis with no warranty expressed or implied.  CL grants permission to use
// this software but offers no guarantees regarding its usefulness toward any
// application.
// 
// gba.v 
// 
// Log:
// Revision 1.0: 7/16/02 by Rich LeGrand (rich@charmedlabs.com)
// Initial release
// 

module Gba(Clk, AddrDataIn, AddrIn, AddrOut, Ccs, Crd, Cwr, Fce, Foe, Fwe, Ireq, P);

   parameter ADDR_LOGIC = 12'hfff;  
         
   input  Clk;
   inout  [15:0] AddrDataIn;  
   input  [7:0] AddrIn;
   output [23:0] AddrOut;
   input  Ccs;
   input  Crd;
   input  Cwr;
   output Fce;
   output Foe;
   output Fwe;
   output Ireq;
   output [54:0] P;
         
   reg     [15:0] AddrDataIn;      	  
   reg     [23:0] AddrOut;  
   reg     Fce;
   reg     Foe;
   reg     Fwe;
   reg     ClkAddr;
   reg     BusState;
   reg     En;
   reg     Lrd;
   reg     Lwr;
   integer i;
   
   always @(AddrOut or Ccs)
      begin
      if (Ccs==1'b0)
         En = AddrOut[23:12]==ADDR_LOGIC; 
      else 
         En = 1'b0;
      end
               
   always @(Ccs or Crd or Cwr or En)
      begin
      Fce = Ccs | En;		// asserted low
      Foe = Ccs | En | Crd;	// asserted low	
      Fwe = Ccs | En | Cwr;	// asserted low
      Lwr = En & ~Ccs & ~Cwr;	// asserted high
      Lrd = En & ~Ccs & ~Crd;	// asserted high
      ClkAddr = ~Ccs & Crd; 
      end
   
   always @(negedge ClkAddr or posedge Ccs)
      begin
      if (Ccs)
         BusState = 1'b0;
      else
         BusState = 1'b1;
      end
       
   always @(posedge ClkAddr)
      begin
      if (BusState)
         AddrOut = AddrOut + 1;
      else
         AddrOut = {AddrIn, AddrDataIn}; 
      end         	
   
   LogicMap Inst1LogicMap(AddrOut, AddrDataIn, En, Lrd, Lwr, Clk, Ireq, P);
   
endmodule

