//
// This software is provided for free by Danny H. Newport on an "as is" basis with
// no warranty expressed or implied.  I grant permission to use this software with 
// the understanding that there are no guarantees regarding its usefulness for any
// application.
//
// Danny H. Newport
// www.danny-newport.com
// 
// botbgba.cxx 
// 
// Log:
// Revision 1.1: 8-25-02 by Danny H. Newport (dannynewp@aol.com)
// Initial release
// Compiled with xport's gcc arm compiler (www.charmedlabs.com) under the projects directory
// named botbgba, but should be able to compile under any gcc with
// maybe some modifications.
//
// please report any bugs, wishes, improvements....etc.
// 

#include "../../include/textdisp.h"
#include "../../include/gba.h"
#include "../../gcc-arm/include/string.h"

extern "C" 
{
	int Main(void);
}


void CheckGuess();		//function to check if guess is correct
void ClearMidScreen();	//clears the middle of the screen
void DispayTitle();		//displays program title screen
void DisplayHelp();		//displays a little bit of help
void DisplayHint();		//displays the hint
void DisplayMain();		//displays the main program text
void DisplayMenu();		//displays the 'select' menu
void DisplayReset();	//displays reset screen
int GetKey();			//wait for a button to be pressed and returns a number
int GetKeyNoWait();		//check for a button being pressed and returns result
void GotoABook();		//displays goto book screen
void MainLoop();		//main display function
void WaitKey();			//wait for a key to be pressed
void WaitPadup();		//wait for buttons to be cleared

//	bible			- correct order
//	book_no			- old gameboy number display.....never converted it yet to gba with just printf
//	bible_guess		- alphabetical list of books
//  bible_hint		- hint for correct order of books
char * bible[]={"Genesis","Exodus","Leviticus","Numbers","Deuteronomy","Joshua","Judges","Ruth","I Samuel","II Samuel","I Kings","II Kings","I Chronicles","II Chronicles",
"Ezra","Nehemiah","Esther","Job","Psalms","Proverbs","Ecclesiastes","Song of Solomon","Isaiah","Jeremiah","Lamentations","Ezekiel","Daniel","Hosea","Joel",
"Amos","Obadiah","Jonah","Micah","Nahum","Habukuk","Zephaniah","Haggai","Zechariah","Malachi","Matthew","Mark","Luke","John","Acts","Romans","I Corinthians",
"II Corinthians","Galatians","Ephesians","Philippians","Colossians","I Thessalonians","II Thessalonians","I Timothy","II Timothy","Titus","Philemon","Hebrews",
"James","I Peter","II Peter","I John","II John","III John","Jude","Revelation"};
char * book_no[]={"1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24","25","26","27","28","29","30","31","32","33","34",
"35","36","37","38","39","40","41","42","43","44","45","46","47","48","49","50","51","52","53","54","55","56","57","58","59","60","61","62","63","64","65","66"};
char * bible_guess[]={"Acts","Amos","Colossians","Daniel","Deuteronomy","Ecclesiastes","Ephesians","Esther","Exodus","Ezekiel","Ezra","Galatians","Genesis","Habakkuk","Haggai",
"Hebrews","Hosea","I Chronicles","I Corinthians","I John","I Kings","I Peter","I Samuel","I Thessalonians","I Timothy","II Chronicles","II Corinthians","II John",
"II Kings","II Peter","II Samuel","II Thessalonians","II Timothy","III John","Isaiah","James","Jeremiah","Job","Joel","John","Jonah","Joshua","Jude","Judges",
"Lamentations","Leviticus","Luke","Malachi","Mark","Matthew","Micah","Nahum","Nehemiah","Numbers","Obadiah","Philemon","Philippians","Proverbs","Psalms","Revelation",
"Romans","Ruth","Song of Solomon","Titus","Zechariah","Zephaniah"};
char * bible_hint[]={"G","E","L","N","D","J","J","R","I S","II S","I K","II K","I C","II C","E","N","E","J","P","P","E","S","I","J","L","E","D","H","J","A","O",                                                 "J","M","N","H","Z","H","Z", "M" , "M","M","L","J","A","R","I C","II C","G","E","P","C","I T","II T","I T","II T","T","P","H","J","I P","II P","I J", "II J","III J","J","R"};

CTextDisp td;			//display screen handle (may not need it?)
int strcomp;			//compare correct answer with guess
int guess_no;			//keep track of current guess count
int guess_max;			//set maximum user guesses before counting wrong
int correct_cnt;		//keep track of user's correct answer count
int gameover;			//check to see if game is over
int score ;				//keep track of user's score
int current_bk_no;		//keep track of current book number
int loop;				//used for 'for' loops
int a;					//used for keeping track of which key was pressed
int temp;				//temp storage location
int temp2;				//temp 2 storage location
int cmpare;				//string compare result
int books_guess_no;		//current book guess count
int options;			//user option storage
int menuitem;			//menu item selected
int showhint;			//should a hint be shown?  1 if it should be
char * test1;			//string storage 1 for guess compare
char * test2;			//string storage 2 for guess compare

int Main(void)
{
	
	//set initial settings
	books_guess_no=0;
	current_bk_no=0;
	guess_no=1;
	guess_max=3;
	correct_cnt=0;
	gameover=0;
	score=0;
	options=1;
	showhint=0;
	
	DispayTitle();
	a=GetKeyNoWait();
	while(a!=4)			//wait for start key
	{
		a=GetKeyNoWait();
	}
	
	DisplayHelp();
	a=GetKeyNoWait();
	while(a!=4)			//wait for start key
	{
		a=GetKeyNoWait();
	}
	
	td.Clear();
	while(1)
	{	
		DisplayMain();
		
		if(showhint==1)
		{
			DisplayHint();			
		}
		
		MainLoop();		
	}
	
	//this section should never get executed....
	td.SetPosition(0,0);
	td.Printf("     ALERT!  YOU SHOULD NEVER\nGET HERE\n\n");
	
	WaitKey();
	while(1)
	{
		WaitPadup();
	}
}


void MainLoop()
{
	
	WaitKey();
	switch(GetKey())
	{
		
	case 1://key a
		showhint=0;
		CheckGuess();
		break;
	case 2://key b
		if(gameover!=1)
			showhint=1;
		WaitPadup();
		break;
	case 3://key select
		showhint=0;
		DisplayMenu();
		WaitPadup();
		break;
	case 4://key start
		showhint=0;
		DisplayReset();
		break;
	case 5://joykey right
		showhint=0;
		if(books_guess_no<58)
			books_guess_no+=8;
		else books_guess_no=0;
		WaitPadup();
		break;
	case 6://joykey left
		showhint=0;
		if(books_guess_no>8)
			books_guess_no-=8;
		else books_guess_no=0;
		WaitPadup();
		break;
	case 7://joykey up
		showhint=0;
		if(books_guess_no<65)
			books_guess_no++;
		else books_guess_no=0;
		WaitPadup();
		break;
	case 8://joykey down
		showhint=0;
		if(books_guess_no>0)
			books_guess_no--;
		else books_guess_no=65;
		WaitPadup();
		break;
	case 9://key r - what use can this be for???
		showhint=0;
		break;
	case 10://key l - what use can this be for???
		showhint=0;
		break;
	default:
		CheckGuess();
		break;
		
	}
	
}

void CheckGuess()
{
	td.Clear();
	strcomp = 5;
	test1 = "test1";
	test2 = "test2";
	td.SetPosition(0,0);
	td.Printf("Books of the Bible");
	test1 = bible_guess[books_guess_no];
	test2 = bible[current_bk_no];
	strcomp = stricmp(test1,test2);	
	if(gameover==1)
	{
		ClearMidScreen();
		td.SetPosition(0,11);
		td.Printf("Game Over!\nPress Start Button\nto start over\n");
		
	}
	
	if((strcomp<0||strcomp>0)&&(current_bk_no<66)&&(gameover!=1))
	{	
		if(guess_no<=guess_max)
		{
			guess_no++;
			ClearMidScreen();
			td.SetPosition(0,9);
			td.Printf("Sorry, try again...      \n");
			
		} 
		
		if(guess_no>guess_max)
		{
			
			td.SetPosition(0,9);
			td.Printf("Nope! Answer was...      \n");
			guess_no=1;
			td.Printf(bible[current_bk_no]);
			current_bk_no++;
			score-=5;
			td.SetPosition(0,0);
		}
		
		
		if(gameover==1)
		{
			ClearMidScreen();
			td.SetPosition(0,11);
			td.Printf("Game Over!\nPress Start Button\nto start over\n");                                                                            
		}      
		
	}
	else if(strcomp==0&&(current_bk_no<66&&current_bk_no>=0)&&(gameover!=1))
	{
		
		current_bk_no++;
		td.SetPosition(0,8);
		td.Printf("Correct!!....     \n                   \n                   \n                   \n                   \n");
		td.SetPosition(0,9);
		td.Printf(bible[current_bk_no-1]);
		guess_no=1;
		correct_cnt++;
		score+=10;
		td.SetPosition(0,0);
		
	}
	
	if(current_bk_no==66)
	{
		gameover=1;
		td.SetPosition(0,11);
		td.Printf("                   \n                   \n                   \n                   \n");
		td.SetPosition(0,11);                                                                      
		td.Printf("Game Over!\nPress Start Button\nto start over\n");                                                                            
		
	}	
	
	td.SetPosition(0,15);
	td.Printf("Guess count:%d  \n",guess_no);
	td.Printf("Correct count:%d  \n",correct_cnt);
	
	td.SetPosition(0,0);
	
	return;
	
}

void WaitKey()//wait for key to be raised
{
	do
	{
		switch(GBA_REG_P1)
		{
		case 0x3ff:
			a=0;
			break;
		default:
			a=GetKey();
			break;
		}
	}while(a!=0);
	WaitPadup();
	return;
}

void DispayTitle()
{
	
	td.Printf("     Books of the Bible\n     for Gameboy Advance\n\n");
	td.Printf("    by: Danny H. Newport");
	td.Printf("\n\n\n\n\n\n\nE-mail:    dannynewp@aol.com");
	td.Printf("\nHome Page: http://www.danny-newport.com");
	td.Printf("\n\n\n\n\n\n\n      Press Start\n");
	WaitPadup();
	
	return;	
	
}

void DisplayHelp()
{
	td.Clear();
	td.SetPosition(0,0);
	td.Printf("     Books of the Bible\n     for Gameboy Advance\n");
	td.Printf("\nA Btn - Check ans");
	td.Printf("\nB Btn - Hint");
	td.Printf("\nUp    - Book Up");
	td.Printf("\nDown  - Book Dwn");
	td.Printf("\nRight - Page Up");
	td.Printf("\nLeft  - Page Down");
	td.Printf("\nStart - Reset");
	td.Printf("\nSelect- Options");
	td.Printf("\n\n\n\nPress Start");
	WaitPadup();
	
	return;
	
}

void DisplayHint()
{
	
	ClearMidScreen();
	td.SetPosition(0,11);
	if(gameover==0)
	{
		td.Printf("Hint: ");
		td.Printf("\nStarts with: ");
		td.Printf(bible_hint[current_bk_no]);
		score-=1;
	}
	else
	{
		td.Printf("\nGame Over!\nPress Start Button\nto start over\n");
	}
	
	return;
	
}


void DisplayReset()
{
	
	ClearMidScreen();
	td.SetPosition(0,8);
	td.Printf("Reset game?");
	td.Printf("\nA - Reset\nB - Cancel");
	a=0;
	temp=0;
	while(temp!=99)
	{
		a=GetKeyNoWait();
		if(a==1||a==2)
			temp=99;
	}
	
	
	if(a==1)
	{
		books_guess_no=1;
		current_bk_no=0;
		guess_no=1;				
		correct_cnt=0;
		books_guess_no=0;
		gameover=0;
		score=0;
		ClearMidScreen();
		td.SetPosition(0,3);
		td.Printf("Enter Book #: 1  ");  
		
	}
	else if(a==2)
	{
		ClearMidScreen();
		
	}
	
	
	return;
	
}

void DisplayMain()
{
	
	if(current_bk_no>65)
		current_bk_no=65;
	td.SetPosition(0,0);
	td.Printf("     Books of the Bible\n     for Gameboy Advance\n\n");
	td.SetPosition(0,3);
	td.Printf("Enter Book #: %s",book_no[current_bk_no]);
	td.SetPosition(0,5);
	td.Printf("                         ");//erase old book
	td.SetPosition(0,5);
	td.Printf(bible_guess[books_guess_no]);
	td.SetPosition(0,15);
	td.Printf("Guess count:%d\n",guess_no);
	td.Printf("Correct count:%d\n",correct_cnt);
	td.SetPosition(0,17);
	td.Printf("Score : %d\n",score);
	td.SetPosition(0,19);
	td.Printf("by:Danny H. Newport");
	
	
	return;
	
}

void ClearMidScreen()
{
	
	td.SetPosition(0,6);
	td.Printf("                   \n                   \n                   \n");
	td.Printf("                   \n                   \n                   \n");
	td.Printf("                   \n                   \n");
	
}

void DisplayMenu()
{
	
	menuitem = 6;
	td.Clear();	
	
	do
	{
		td.SetPosition(0,0);
		td.Printf("     Books of the Bible\n     for Gameboy Advance\n\n");
		
		switch(options)
		{
		case 0:
			break;
		case 1:
			menuitem = 1;
			td.SetPosition(0,4);
			td.Printf("--> start at NT");
			td.SetPosition(0,5);
			td.Printf("    start at OT");
			td.SetPosition(0,6);
			td.Printf("    goto a book");
			td.SetPosition(0,7);
			td.Printf("    view help  ");
			td.SetPosition(0,8);
			td.Printf("    Exit       ");
			break;
			
		case 2:
			menuitem = 2;
			td.SetPosition(0,4);
			td.Printf("    start at NT");
			td.SetPosition(0,5);
			td.Printf("--> start at OT");
			td.SetPosition(0,6);
			td.Printf("    goto a book");
			td.SetPosition(0,7);
			td.Printf("    view help  ");
			td.SetPosition(0,8);
			td.Printf("    Exit       ");
			break;
			
		case 3:
			menuitem = 3;
			td.SetPosition(0,4);
			td.Printf("    start at NT");
			td.SetPosition(0,5);
			td.Printf("    start at OT");
			td.SetPosition(0,6);
			td.Printf("--> goto a book");
			td.SetPosition(0,7);
			td.Printf("    view help  ");
			td.SetPosition(0,8);
			td.Printf("    Exit       ");
			
			break;
		case 4:
			menuitem = 4;
			td.SetPosition(0,4);
			td.Printf("    start at NT");
			td.SetPosition(0,5);
			td.Printf("    start at OT");
			td.SetPosition(0,6);
			td.Printf("    goto a book");
			td.SetPosition(0,7);
			td.Printf("--> view help  ");
			td.SetPosition(0,8);
			td.Printf("    Exit       ");
			
			break;
		case 5:
			menuitem=5;
			td.SetPosition(0,4);
			td.Printf("    start at NT");
			td.SetPosition(0,5);
			td.Printf("    start at OT");
			td.SetPosition(0,6);
			td.Printf("    goto a book");
			td.SetPosition(0,7);
			td.Printf("    view help  ");
			td.SetPosition(0,8);
			td.Printf("--> Exit       ");
			break;	
		}
		temp = 0;
		while(temp!=99)
		{
			switch(GetKeyNoWait())
			{
			case 1:
				temp=99;
				a=1;
				break;
			case 2:
				break;
			case 8:
				if(options<5)
					options++;
				else options = 1;
				temp=99;
				break;
			case 7:
				if(options>1)
					options--;	
				else options = 5;
				temp=99;
				break;
			}
		}
		
	}
	while(a!=1);
	
	
	switch(menuitem)
	{
	case 0://view score - not used....put it on main screen
		break;
	case 1://start at NT
		current_bk_no = 39;
		break;
	case 2://start at OT
		current_bk_no = 0;
		break;
	case 3://goto a book
		GotoABook();
		WaitPadup();
		break;
	case 4://view help
		DisplayHelp();
		a=GetKeyNoWait();
		while(a!=4)
		{
			a=GetKeyNoWait();
		}
		break;
	case 5:
		break;
	}
	
	WaitPadup();	
	td.Clear();
	
	return;
	
}

int GetKey()
{
	
	a=0;
	while(1)
	{
		switch(GBA_REG_P1)
		{
		case 0x3fe://key A
			a=1;
			break;
		case 0x3fd://key B
			a=2;
			break;
		case 0x3fb://key select
			a=3;
			break;
		case 0x3f7://key start
			a=4;
			break;
		case 0x3ef://joykey right
			a=5;
			break;
		case 0x3df://joykey left
			a=6;
			break;
		case 0x3bf://joykey up
			a=7;
			break;
		case 0x37f://Joykey down
			a=8;
			break;
		case 0x2ff://key R
			a=9;
			break;
		case 0x1ff://key L
			a=10;
			break;
		default://0x3ff address is no key pressed
			a=0;
			break;
		}
		if(a>0&&a<11)
			return a;
	}
	
	return a;	//just in case
	
	
}

int GetKeyNoWait()
{
	
	switch(GBA_REG_P1)
	{
	case 0x3fe://key A
		a=1;
		WaitPadup();
		return 1;
		break;
	case 0x3fd://key B
		a=2;
		WaitPadup();
		return 2;
		break;
	case 0x3fb://key select
		a=3;
		WaitPadup();
		return 3;
		break;
	case 0x3f7://key start
		a=4;
		WaitPadup();
		return 4;
		break;
	case 0x3ef://joykey right
		a=5;
		WaitPadup();
		return 5;
		break;
	case 0x3df://joykey left
		a=6;
		WaitPadup();
		return 6;
		break;
	case 0x3bf://joykey up
		a=7;
		WaitPadup();
		return 7;
		break;
	case 0x37f://Joykey down
		a=8;
		WaitPadup();
		return 8;
		break;
	case 0x2ff://key R
		a=9;
		WaitPadup();
		return 9;
		break;
	case 0x1ff://key L
		a=10;
		WaitPadup();
		return 10;
		break;
	default://0x3ff address is no key pressed
		return 0;
		break;
	}
	
}

void WaitPadup()
{
	
	do
	{
		switch(GBA_REG_P1)
		{
		case 0x3ff:
			a=0;
			break;
		default:
			a=12;
			break;
		}
	}while(a!=0);
	
	for(loop=0;loop<10000;loop++);
	
	return;
	
}


void GotoABook()
{
	
	temp2=current_bk_no;
	td.Clear();
	ClearMidScreen();
	
	do
	{
		td.SetPosition(0,0);
		td.Printf("     Books of the Bible\n     for Gameboy Advance\n\n");
		td.SetPosition(0,6);
		td.Printf("press A button when done");
		td.SetPosition(0,8);
		td.Printf("please enter a book from \n1 to 66 \n\nPress B to reset book\n\n");
		td.Printf("goto book #: %s",book_no[current_bk_no]);
		
		temp = 0;
		while(temp!=99)
		{
			switch(GetKeyNoWait())
			{
			case 1://enter the book
				temp=99;
				a=1;
				break;
			case 2:
				current_bk_no=temp2;
				a=2;
				temp=99;
				break;
			case 5://right
				if(current_bk_no<=55)
					current_bk_no+=10;
				temp=99;
				break;
			case 6://left
				if(current_bk_no>=10)
					current_bk_no-=10;
				temp=99;
				break;
			case 7://up
				if(current_bk_no<64)
					current_bk_no++;
				else current_bk_no = 0;
				temp=99;
				break;
			case 8://down
				if(current_bk_no>1)
					current_bk_no--;	
				else current_bk_no = 65;
				temp=99;
				break;
			case 9://key R
				if(current_bk_no<=55)
					current_bk_no+=10;
				temp=99;
				break;
			case 10://key L
				if(current_bk_no>=10)
					current_bk_no-=10;
				temp=99;
				break;
			}
		}
		
	}
	while(a!=1);
	
	
	WaitPadup();	
	td.Clear();
	
	
	return;
	
	
}

